@@ -36,6 +36,7 @@
 #include <lsmmathmloperatorelement.h>
 #include <lsmmathmlstringelement.h>
 #include <lsmmathmlrowelement.h>
+#include <lsmmathmlencloseelement.h>
 #include <lsmmathmlpaddedelement.h>
 #include <lsmmathmlfencedelement.h>
 #include <lsmmathmlerrorelement.h>
@@ -102,6 +103,8 @@ _create_element (LsmDomDocument *document, const char *tag_name)
 		node = lsm_mathml_operator_element_new ();
 	else if (strcmp (tag_name, "mrow") == 0)
 		node = lsm_mathml_row_element_new ();
+	else if (strcmp (tag_name, "menclose") == 0)
+		node = lsm_mathml_enclose_element_new ();
 	else if (strcmp (tag_name, "mn") == 0)
 		node = lsm_mathml_number_element_new ();
 	else if (strcmp (tag_name, "mi") == 0)
@@ -213,16 +216,14 @@ typedef enum {
 } LsmMathmlDocumentError;
 
 LsmMathmlDocument *
-lsm_mathml_document_new_from_itex (const char *itex, int size, GError **error)
+lsm_mathml_document_new_from_itex (const char *itex, gssize size, GError **error)
 {
 	LsmDomDocument *document;
-	char *xml;
+	char *mathml;
 
-	g_return_val_if_fail (itex != NULL, NULL);
+	mathml = lsm_itex_to_mathml (itex, size);
 
-	xml = itex2MML_parse (itex, size);
-
-	if (xml == NULL) {
+	if (mathml == NULL) {
 		lsm_debug_dom ("[LsmMathmlDocument::new_from_itex] Invalid document");
 
 		g_set_error (error,
@@ -233,9 +234,9 @@ lsm_mathml_document_new_from_itex (const char *itex, int size, GError **error)
 		return NULL;
 	}
 
-	document = lsm_dom_document_new_from_memory (xml, -1, error);
+	document = lsm_dom_document_new_from_memory (mathml, -1, error);
 
-	itex2MML_free_string (xml);
+	lsm_itex_free_mathml_buffer (mathml);
 
 	if (document == NULL)
 		return NULL;
@@ -252,7 +253,7 @@ static LsmMathmlDocument *
 lsm_mathml_document_new_from_itex_file (GFile *file, GError **error)
 {
 	LsmMathmlDocument *document;
-	gsize size = 0;
+	gssize size = 0;
 	char *contents = NULL;
 
 	if (!g_file_load_contents (file, NULL, &contents, &size, NULL, error))
