@@ -1,4 +1,5 @@
-/*
+/* Lasem
+ *
  * Copyright © 2009 Emmanuel Pacaud
  *
  * This library is free software; you can redistribute it and/or
@@ -250,6 +251,30 @@ const LsmTraitClass lsm_mathml_form_trait_class = {
 	.to_string = lsm_mathml_form_trait_to_string
 };
 
+static gboolean
+lsm_mathml_notation_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmMathmlNotation *value = (LsmMathmlNotation *) abstract_trait;
+
+	*value = lsm_mathml_notation_from_string (string);
+
+	return *value >= 0;
+}
+
+static char *
+lsm_mathml_notation_trait_to_string (LsmTrait *abstract_trait)
+{
+	LsmMathmlNotation *value = (LsmMathmlNotation *) abstract_trait;
+
+	return g_strdup (lsm_mathml_notation_to_string (*value));
+}
+
+const LsmTraitClass lsm_mathml_notation_trait_class = {
+	.size = sizeof (int),
+	.from_string = lsm_mathml_notation_trait_from_string,
+	.to_string = lsm_mathml_notation_trait_to_string
+};
+
 typedef int (*LsmMathmlEnumFromString) (const char *string);
 typedef char * (*LsmMathmlEnumToString) (unsigned int value);
 
@@ -739,7 +764,7 @@ const LsmTraitClass lsm_mathml_space_trait_class = {
 	.to_string = lsm_mathml_space_trait_to_string
 };
 
-gboolean
+static gboolean
 lsm_mathml_space_list_trait_from_string (LsmTrait *abstract_trait, char *string)
 {
 	LsmMathmlSpaceList *space_list = (LsmMathmlSpaceList *) abstract_trait;
