@@ -24,7 +24,7 @@
 #include <lsmdebug.h>
 #include <lsmattributes.h>
 #include <lsmproperties.h>
-#include <lsmdomdocument.h>
+#include <lsmsvgdocument.h>
 #include <lsmsvgelement.h>
 #include <lsmsvgtransformable.h>
 #include <lsmsvgpatternelement.h>
@@ -70,22 +70,28 @@ lsm_svg_element_set_attribute (LsmDomElement *self, const char* name, const char
 	lsm_debug_dom ("[LsmSvgElement::set_attribute] node = %s, name = %s, value = %s",
 		    lsm_dom_node_get_node_name (LSM_DOM_NODE (self)), name, value);
 
-	/* TODO Avoid double hash table lookup */
-	if (!lsm_attribute_manager_set_attribute (s_element_class->attribute_manager,
-						  self, name, value))
-		lsm_svg_property_bag_set_property (&s_element->property_bag, name, value);
-
 	if (g_strcmp0 (name, "id") == 0 ||
 	    g_strcmp0 (name, "xml:id") == 0) {
 		LsmDomDocument *document;
 
 		document = lsm_dom_node_get_owner_document (LSM_DOM_NODE (self));
 		if (document != NULL)
-			lsm_dom_document_register_element (document, LSM_DOM_ELEMENT (self), value);
+			lsm_svg_document_register_element (LSM_SVG_DOCUMENT (document), LSM_SVG_ELEMENT (self),
+							   value, s_element->id.value);
+
+		lsm_attribute_manager_set_attribute (s_element_class->attribute_manager,
+						     self, name, value);
+
+		return;
 	}
+
+	/* TODO Avoid double hash table lookup */
+	if (!lsm_attribute_manager_set_attribute (s_element_class->attribute_manager,
+						  self, name, value))
+		lsm_svg_property_bag_set_property (&s_element->property_bag, name, value);
 }
 
-const char *
+static const char *
 lsm_svg_element_get_attribute (LsmDomElement *self, const char *name)
 {
 	LsmSvgElementClass *s_element_class = LSM_SVG_ELEMENT_GET_CLASS(self);
@@ -160,6 +166,7 @@ _transformed_render (LsmSvgElement *element, LsmSvgView *view)
 
 	parent_style = lsm_svg_view_get_current_style (view);
 	style = lsm_svg_style_new_inherited (parent_style, &element->property_bag);
+	style->ignore_group_opacity = element_class->is_shape_element;
 
 	if (style->visibility->value == LSM_SVG_VISIBILITY_VISIBLE &&
 	    style->display->value != LSM_SVG_DISPLAY_NONE) {
