@@ -39,22 +39,43 @@ lsm_svg_line_element_get_node_name (LsmDomNode *node)
 
 /* LsmSvgGraphic implementation */
 
+static void
+_normalize_length (LsmSvgLineElement *line, LsmSvgView *view, double *x1, double *y1, double *x2, double *y2)
+{
+	*x1 = lsm_svg_view_normalize_length (view, &line->x1.length, LSM_SVG_LENGTH_DIRECTION_HORIZONTAL);
+	*y1 = lsm_svg_view_normalize_length (view, &line->y1.length, LSM_SVG_LENGTH_DIRECTION_VERTICAL);
+	*x2 = lsm_svg_view_normalize_length (view, &line->x2.length, LSM_SVG_LENGTH_DIRECTION_HORIZONTAL);
+	*y2 = lsm_svg_view_normalize_length (view, &line->y2.length, LSM_SVG_LENGTH_DIRECTION_VERTICAL);
+
+}
+
 static void
 lsm_svg_line_element_render (LsmSvgElement *self, LsmSvgView *view)
 {
 	LsmSvgLineElement *line = LSM_SVG_LINE_ELEMENT (self);
 	double x1, y1, x2, y2;
 
-	x1 = lsm_svg_view_normalize_length (view, &line->x1.length, LSM_SVG_LENGTH_DIRECTION_HORIZONTAL);
-	y1 = lsm_svg_view_normalize_length (view, &line->y1.length, LSM_SVG_LENGTH_DIRECTION_VERTICAL);
-	x2 = lsm_svg_view_normalize_length (view, &line->x2.length, LSM_SVG_LENGTH_DIRECTION_HORIZONTAL);
-	y2 = lsm_svg_view_normalize_length (view, &line->y2.length, LSM_SVG_LENGTH_DIRECTION_VERTICAL);
+	_normalize_length (line, view, &x1, &y1, &x2, &y2);
 
 	lsm_debug_render ("[LsmSvgLineElement::render] %g, %g, %g, %g", x1, y1, x2, y2);
 
 	lsm_svg_view_show_line (view, x1, y1, x2, y2);
 }
 
+static void
+lsm_svg_line_element_get_extents (LsmSvgElement *self, LsmSvgView *view, LsmExtents *extents)
+{
+	LsmSvgLineElement *line = LSM_SVG_LINE_ELEMENT (self);
+	double x1, y1, x2, y2;
+
+	_normalize_length (line, view, &x1, &y1, &x2, &y2);
+
+	extents->x1 = MIN (x1, x2);
+	extents->y1 = MIN (y1, y2);
+	extents->x2 = MAX (x1, x2);
+	extents->y2 = MAX (y1, y2);
+}
+
 /* LsmSvgLineElement implementation */
 
 LsmDomNode *
@@ -119,8 +140,11 @@ lsm_svg_line_element_class_init (LsmSvgLineElementClass *s_rect_class)
 		LSM_SVG_ELEMENT_CATEGORY_BASIC_SHAPE;
 
 	s_element_class->render = lsm_svg_line_element_render;
+	s_element_class->get_extents = lsm_svg_line_element_get_extents;
 	s_element_class->attribute_manager = lsm_attribute_manager_duplicate (s_element_class->attribute_manager);
 
+	s_element_class->is_shape_element = TRUE;
+
 	lsm_attribute_manager_add_attributes (s_element_class->attribute_manager,
 					      G_N_ELEMENTS (lsm_svg_line_element_attribute_infos),
 					      lsm_svg_line_element_attribute_infos);
