@@ -41,7 +41,7 @@ lsm_svg_blending_mode_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_blending_mode_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgBlendingMode *trait = (LsmSvgBlendingMode *) abstract_trait;
@@ -65,7 +65,7 @@ lsm_svg_comp_op_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_comp_op_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgCompOp *trait = (LsmSvgCompOp *) abstract_trait;
@@ -89,7 +89,7 @@ lsm_svg_enable_background_trait_from_string (LsmTrait *abstract_trait, char *str
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_enable_background_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgEnableBackground *trait = (LsmSvgEnableBackground *) abstract_trait;
@@ -564,7 +564,7 @@ lsm_svg_paint_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return TRUE;
 }
 
-char *
+static char *
 lsm_svg_paint_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgPaint *paint = (LsmSvgPaint *) abstract_trait;
@@ -598,7 +598,7 @@ lsm_svg_fill_rule_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_fill_rule_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgFillRule *trait = (LsmSvgFillRule *) abstract_trait;
@@ -622,7 +622,7 @@ lsm_svg_font_style_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_font_style_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgFontStyle *trait = (LsmSvgFontStyle *) abstract_trait;
@@ -646,7 +646,7 @@ lsm_svg_font_stretch_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_font_stretch_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgFontStretch *trait = (LsmSvgFontStretch *) abstract_trait;
@@ -673,7 +673,7 @@ lsm_svg_font_weight_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 100 && *trait <= 1000;
 }
 
-char *
+static char *
 lsm_svg_font_weight_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgFontWeight *trait = (LsmSvgFontWeight *) abstract_trait;
@@ -703,7 +703,7 @@ lsm_svg_line_join_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_line_join_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgLineJoin *trait = (LsmSvgLineJoin *) abstract_trait;
@@ -888,7 +888,7 @@ lsm_svg_display_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_display_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgDisplay *trait = (LsmSvgDisplay *) abstract_trait;
@@ -945,7 +945,7 @@ lsm_svg_marker_units_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_marker_units_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgMarkerUnits *trait = (LsmSvgMarkerUnits *) abstract_trait;
@@ -969,7 +969,7 @@ lsm_svg_pattern_units_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_pattern_units_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgPatternUnits *trait = (LsmSvgPatternUnits *) abstract_trait;
@@ -1015,7 +1015,7 @@ lsm_svg_preserve_aspect_ratio_trait_from_string (LsmTrait *abstract_trait, char
 	return TRUE;
 }
 
-char *
+static char *
 lsm_svg_preserve_aspect_ratio_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgPreserveAspectRatio *trait = (LsmSvgPreserveAspectRatio *) abstract_trait;
@@ -1101,7 +1101,7 @@ lsm_svg_text_anchor_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_text_anchor_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgTextAnchor *trait = (LsmSvgTextAnchor *) abstract_trait;
@@ -1125,7 +1125,7 @@ lsm_svg_visibility_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_visibility_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgVisibility *trait = (LsmSvgVisibility *) abstract_trait;
@@ -1192,7 +1192,7 @@ lsm_svg_overflow_trait_from_string (LsmTrait *abstract_trait, char *string)
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_overflow_trait_to_string (LsmTrait *abstract_trait)
 {
 	LsmSvgOverflow *trait = (LsmSvgOverflow *) abstract_trait;
@@ -1209,17 +1209,17 @@ const LsmTraitClass lsm_svg_overflow_trait_class = {
 static gboolean
 lsm_svg_writing_mode_trait_from_string (LsmTrait *abstract_trait, char *string)
 {
-	LsmSvgDisplay *trait = (LsmSvgDisplay *) abstract_trait;
+	LsmSvgWritingMode *trait = (LsmSvgWritingMode *) abstract_trait;
 
 	*trait = lsm_svg_writing_mode_from_string (string);
 
 	return *trait >= 0;
 }
 
-char *
+static char *
 lsm_svg_writing_mode_trait_to_string (LsmTrait *abstract_trait)
 {
-	LsmSvgDisplay *trait = (LsmSvgDisplay *) abstract_trait;
+	LsmSvgWritingMode *trait = (LsmSvgWritingMode *) abstract_trait;
 
 	return g_strdup (lsm_svg_writing_mode_to_string (*trait));
 }
